# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CorporateActionCreateParams"]


class CorporateActionCreateParams(TypedDict, total=False):
    ids: Required[List[str]]
    """The requested list of security identifiers.

    Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet
    Permanent Ids. Requests are limited to 50 IDs.
    """

    batch: Literal["Y", "N"]
    """
    Enables the ability to asynchronously "batch" the request, supporting a
    long-running request for up to 20 minutes. Upon requesting batch=Y, the service
    will respond back with an HTTP Status Code of 202. Once a batch request is
    submitted, use batch status to see if the job has been completed. Once
    completed, retrieve the results of the request via batch-result. When using
    Batch, ids limit is increased to 10000 ids per request, though limits on query
    string via GET method still apply. It's advised to submit large lists of ids via
    POST method. Please note that the number of unique currencies present in the
    requested ids is limited to 50 per request.
    """

    cancelled_dividend: Annotated[Literal["include", "exclude"], PropertyInfo(alias="cancelledDividend")]
    """
    The cancelled dividend returns the dividend details whether they are cancelled
    or active.
    """

    currency: str
    """Currency code for adjusting prices.

    Default is Local. For a list of currency ISO codes, visit
    [Online Assistant Page 1470](https://oa.apps.factset.com/pages/1470).
    """

    end_date: Annotated[str, PropertyInfo(alias="endDate")]
    """The end date requested for a given date range in **YYYY-MM-DD** format.

    In the context of corporate actions, this filters the response to only include
    events within the date range. The frequency between the startDate and endDate is
    always set to the "event" frequency- meaning the service will return only events
    within those inclusive boundaries. Leaving both startDate and endDate blank will
    pull "all" events for each requested ids.
    """

    event_category: Annotated[
        Literal["CASH_DIVS", "STOCK_DIST", "RIGHTS", "SPINOFFS", "SPLITS", "ALL"], PropertyInfo(alias="eventCategory")
    ]
    """Selects the Event Category to include in the response.

    - **CASH_DIVS** = Cash Dividends
    - **STOCK_DIST** = Stock Distributions
    - **SPINOFFS** = Spin Offs
    - **RIGHTS** = Rights Issue
    - **SPLITS** = Splits
    - **ALL** = Returns all Event Types. If left blank the service will default to
      ALL.
    """

    fields: List[str]
    """
    Request available Corporate Actions data fields to be included in the response.
    Default is all fields. _fsymId_, _effectiveDate_, _eventTypeCode_ and
    _requestId_ are always included.

    | field         | description                                   |
    | ------------- | --------------------------------------------- |
    | fsymId        | Factset Regional Security Identifier          |
    | eventTypeCode | Character code that denotes the type of event |
    | effectiveDate | The date when security is traded ex-dividend  |
    | requestId     | Identifier that was used for the request.     |

    <h3>Common Fields</h3>
      |field|description|
      |---|---|
      |eventId|Uniquely Identifies the event|
      |eventTypeDesc|Description of the type of event|
      |divTypeCode|Dividend type code. [OA#8764](https://my.apps.factset.com/oa/pages/8764)|
      |announcementDate|Date the event was publicly announced|
      |recordDate|Record date of the event|
      |payDate|Payment date of the event|
    <h3>Dividend Fields</h3>
      |field|description|
      |---|---|
      |currency|Currency ISO code associated with distribution amount converted into trading currency of the record.|
      |amtDefNetGrossIndicator|Indicates whether the default amount is net or gross. G=Gross; N=Net.|
      |amtDefTradingAdj|Cash distribution amount (net or gross) in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is adjusted for splits.|
      |amtDefTradingUnadj|Cash distribution amount (net or gross) in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is not adjusted for splits. |
      |amtNetTradingAdj|Net distribution amount in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date.  The value is adjusted for splits.|
      |amtNetTradingUnadj|Net distribution amount in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is not adjusted for splits.|
      |amtGrossTradingAdj|Gross distribution amount in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is adjusted for splits.|
      |amtGrossTradingUnadj|Gross distribution amount in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is not adjusted for splits.|
      |declaredCurrency|Currency ISO code associated with the declared amount. |
      |amtDefDecAdj|Cash distribution amount (net or gross) in the currency it was declared in. The value is adjusted for splits.|
      |amtDefDecUnadj|Cash distribution amount (net or gross) in the currency it was declared in. The value is not adjusted for splits.|
      |amtNetDecAdj|Net cash distribution amount in the currency it was declared in. The value is adjusted for splits. |
      |amtNetDecUnadj|Net cash distribution amount in the currency it was declared in. The value is not adjusted for splits.|
      |amtGrossDecAdj|Gross cash distribution amount in the currency it was declared in. N/A is returned if the gross amount is not available. The value is adjusted for splits.|
      |amtGrossDecUnadj|Gross cash distribution amount in the currency it was declared in. The value is not adjusted for splits.|
      |dividendStatus|Identifies the cancelled dividents and helps to evaluate their price and portfolio performance.|
      |dividendActiveFlag|Identifies whether the dividend record is currently active(1) or inactive(0).Its applicable to Dividend(DVC) and Dividend with DRP option(DVCD)|
      |dividendsSpecFlag|Indicates a special price implications exists, which may or may not include special dividends. Indicates whether an adjustment should be made to historical pricing.|
      |dividendFrequencyDesc|Dividend Frequency for different event types in the form of a text as per the descriptions found here [OA#8764](https://my.apps.factset.com/oa/pages/8764#Frequency)|
      |dividendFrequencyCode|Dividend Frequency for different event types in the form of a code as per the details found here [OA#8764](https://my.apps.factset.com/oa/pages/8764#Frequency)|
      |frankDefTradingAdj|Split amount of dividend that is franked (subject to tax credit). Published in the trading currency of the input ID.Amount is translated to the trading currency based on the exchange rate as of the effective date. **Only applicable for Australian Securities**.|
      |frankDefTradingUnadj|Unsplit amount of dividend that is franked (subject to tax credit). Published in the trading currency of the input ID.Amount is translated to the trading currency based on the exchange rate as of the effective date. **Only applicable for Australian Securities**.|
      |frankDefDecAdj|Split amount of dividend that is franked (subject to tax credit). Published in the currency the dividend was declared in.**Only applicable for Australian Securities**.|
      |frankDefDecUnadj|Unsplit amount of dividend that is franked (subject to tax credit). Published in the currency the dividend was declared in.**Only applicable for Australian Securities**.|
      |frankPct|Percent of total dividend that is franked (subject to tax credit). **Only applicable for Australian Securities**.|          
      |taxRate|Domestic Withholding Tax Rate for a Resident Individual|
    <h3>Distribution Fields</h3>
      |field|description|
      |---|---|
      |adjFactor|Factor applied to adjust historical prices. Calculation formulas are available on [OA#12619](https://my.apps.factset.com/oa/pages/12619)|
      |adjFactorCombined|Combined adjustment factor for all distribution events on that day.|
      |amtDefTradingAdj|Cash distribution amount (net or gross) in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is adjusted for splits.|
      |amtDefTradingUnadj|Cash distribution amount (net or gross) in the trading currency of the record. Amount is translated to the trading currency based on the exchange rate as of the effective date. The value is not adjusted for splits. |
      |currency|Currency ISO code associated with distribution amount converted into trading currency of the record.|
      |distPct|Distribution percentage of the event  (i.e. 10%). Typical for stock distributions.|
      |distOldTerm|Component of distribution ratio- Number of shares held.|
      |distNewTerm|Component of distribution ratio - Number of shares received.|
      |rightsIssuePrice|Price of the rights issue. |
      |rightsIssueCurrency|Currency the rights issue price was declared in.|
      |shortDesc|Textual description identifying the event. Example- Split (Mandatory): 3 for 1.|
    <h3>Splits Fields</h3>
      |field|description|
      |---|---|
      |adjFactor|Distribution percentage of the event  (i.e. 10%). Typical for stock distributions.|
      |adjFactorCombined|Combined adjustment factor for all distribution events on that day.|
      |distOldTerm|Component of distribution ratio- Number of shares held.|
      |distNewTerm|Component of distribution ratio - Number of shares received.|
      |distInstFsymId|Helps to identify an instrument representing the distributed company or security associated with ca event identifier. Its applicable for types like Bonus issue(BNS),Stock dividend(DVS),Rights issue (DSR), and spin off(SPO). |
      |shortDesc|Textual description identifying the event. Example- Split (Mandatory): 3 for 1.|
    """

    start_date: Annotated[str, PropertyInfo(alias="startDate")]
    """The start date requested for a given date range in **YYYY-MM-DD** format.

    In the context of corporate actions, this filters the response to only include
    events within the date range. The frequency between the startDate and endDate is
    always set to the "event" frequency- meaning the service will return only events
    within those inclusive boundaries. Leaving both startDate and endDate blank will
    pull "all" events for each requested ids.
    """
